==> api-service-common-0.12.28.tgz: package/src/plugins/ua.ts <==
/*************************************************************************
 *
 * Troven CONFIDENTIAL
 * __________________
 *
 *  (c) 2017-2020 Troven Pty Ltd
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Troven Pty Ltd and its licensors,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Troven Pty Ltd
 * and its suppliers and may be covered by International and Regional Patents,
 * patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Troven Pty Ltd.
 */

import * as assert from "assert";
import {IChassisContext, IChassisPlugin, JWT} from "api-service-core";
import * as _ from "lodash";

import UA from 'universal-analytics';
import uuidv5 from 'uuid/v5';

const UA_OPTS = {https: true, strictCidFormat: false };

/**
 * Google Analytics
 *
 * Feature routes local events to the Google Analytics event API
 *
 * @type {{name: string, title: string, see_also: [null], fn: module.exports.fn}}
 */
export default class ua implements IChassisPlugin {
    name = "ua";
    title = "Transmit Universal Analytics to Google";

    see_also: [
        { title: "Docs", url: "https://www.npmjs.com/package/universal-analytics" }
    ];

    install(context: IChassisContext, _options: any) {

        let options: any = _.extend({}, _options);
        assert(options.account, "missing options.account");

        let ctx = (<any>context);
        // add UA visitor factory to chassis context

        ctx.ua = {};
        ctx.ua.visitor = function(req) {
            let jwt = req?JWT.getJWT(context, req):false; // get JWT if req is passed

            if (!jwt) UA( options.account, UA_OPTS); // random visitor

            let visitor_uuid = uuidv5(jwt.sub, uuidv5.DNS); // convert JWT subject to uuid

            return UA( options.account, visitor_uuid, UA_OPTS ); // known visitor
        };

        //  detect a client ID used by client-side tracking

        let client_side = options.client_side;

        if (client_side) {
            let cookieName = _.isString(client_side)?client_side:'_ga';
            context.api.use(UA.middleware(options.account, {cookieName: cookieName}));
        };

        // bind to a express route, if options.path is supplied

        if (options.path) {
            context.api.use(options.path, function(req) {

                let params = {
                    ec: context.config.name,
                    ea: options.action || req.method,
                    // el: msg.message,
                    dp: req.path
                };

                let visitor = ctx.ua.visitor(req);
                visitor.event(params).send();
            });
        }

        // bind context.bus events when options.on is a map of event_name:event_params

        _.each(options.on, function(opts, event) {
            if (!_.isObject(opts)) opts = { ec: options.toString() };

            context.bus.on(event, function(msg) {

                let params = {
                    ec: opts.ec || context.config.name,
                    ea: opts.ea || msg.action || msg.code  || event,
                    ev: opts.ev || msg.value || msg.score || 0,
                    el: opts.el || msg.message || ""
                };

                let visitor = ctx.ua.visitor();
                visitor.event(params).send();
            });
        });

    }
};

==> api-service-common-0.16.13.tgz: package/src/plugins/ua.ts <==
/*************************************************************************
 *
 * Troven CONFIDENTIAL
 * __________________
 *
 *  (c) 2017-2020 Troven Pty Ltd
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Troven Pty Ltd and its licensors,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Troven Pty Ltd
 * and its suppliers and may be covered by International and Regional Patents,
 * patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Troven Pty Ltd.
 */

import * as assert from "assert";
import {IChassisContext, IChassisPlugin, JWT} from "api-service-core";
import * as _ from "lodash";

import UA from 'universal-analytics';

const UA_OPTS = {https: true, strictCidFormat: false };

/**
 * Google Analytics
 *
 * Feature routes local events to the Google Analytics event API
 *
 * @type {{name: string, title: string, see_also: [null], fn: module.exports.fn}}
 */
export default class ua implements IChassisPlugin {
    name = "ua";
    title = "Transmit Universal Analytics to Google";

    see_also: [
        { title: "Docs", url: "https://www.npmjs.com/package/universal-analytics" }
    ];

    install(context: IChassisContext, _options: any) {

        let options: any = _.extend({}, _options);
        assert(options.account, "missing options.account");

        let ctx = (<any>context);
        // add UA visitor factory to chassis context

        ctx.ua = {};
        ctx.ua.visitor = function(req) {
            let jwt = req?JWT.getJWT(context, req):false; // get JWT if req is passed

            if (!jwt) {
                return UA( options.account, UA_OPTS); // random visitor
            } else {
                return UA( options.account, jwt.sub, UA_OPTS ); // known visitor
            }
        };

        //  detect a client ID used by client-side tracking

        let client_side = options.client_side;

        if (client_side) {
            let cookieName = _.isString(client_side)?client_side:'_ga';
            context.api.use(UA.middleware(options.account, {cookieName: cookieName}));
        };

        // bind to a express route, if options.path is supplied

        if (options.path) {
            context.api.use(options.path, function(req) {

                let params = {
                    ec: context.config.name,
                    ea: options.action || req.method,
                    // el: msg.message,
                    dp: req.path
                };

                let visitor = ctx.ua.visitor(req);
                visitor.event(params).send();
            });
        }

        // bind context.bus events when options.on is a map of event_name:event_params

        _.each(options.on, function(opts, event) {
            if (!_.isObject(opts)) opts = { ec: options.toString() };

            context.bus.on(event, function(msg) {

                let params = {
                    ec: opts.ec || context.config.name,
                    ea: opts.ea || msg.action || msg.code  || event,
                    ev: opts.ev || msg.value || msg.score || 0,
                    el: opts.el || msg.message || ""
                };

                let visitor = ctx.ua.visitor();
                visitor.event(params).send();
            });
        });

    }
};
